/*
    $Id: k164d.c,v 1.2 2001/12/01 09:07:26 james Exp $

    k164d, telephone call logger daemon
    Copyright (C) 2001  James Cameron (quozl@us.netrek.org)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef lint
static char vcid[] = "$Id: k164d.c,v 1.2 2001/12/01 09:07:26 james Exp $";
#endif /* lint */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>
#include <ctype.h>
#include <termios.h>
#include <unistd.h>
#include <string.h>
#include <glib.h>

FILE *serial;

static void serial_open(char *device)
{
  int fd;
  struct termios termios;

  serial = fopen(device, "r" );
  if (serial == NULL) {
    char *buffer = g_strdup_printf("fopen: %s", device);
    perror(buffer);
    g_free(buffer);
    exit(1);
  }

  /* set the serial port characteristics */
  fd = fileno(serial);
  if (tcgetattr(fd, &termios) < 0) {
    perror("tcgetattr");
    exit(1);
  }

  if (cfsetospeed(&termios, B9600) < 0) {
    perror("cfsetospeed");
    exit(1);
  }

  if (cfsetispeed(&termios, B9600) < 0) {
    perror("cfsetispeed");
    exit(1);
  }

  if (tcsetattr(fd, TCSANOW, &termios) < 0) {
    perror("tcsetattr");
    exit(1);
  }
}

static void dputs(char *buffer, FILE *file)
{
  int bytes;
  fputs(buffer, stdout);
  bytes = fputs(buffer, file);
  if (bytes == EOF) { perror("fputs"); exit(1); }
}

static void dputc(int byte, FILE *file)
{
  fputc(byte, stdout);
  byte = fputc(byte, file);
  if (byte == EOF) { perror("fputc"); exit(1); }
}

static void dflush(FILE *file)
{
  fflush(stdout);
  fflush(file);
}

static void dputt(time_t now, FILE *file)
{
  char *buffer = g_strdup_printf("%d", (int) now);
  dputs(buffer, file);
  g_free(buffer);
}

time_t to_time(char *p)
{
  /* 01-Dec-01 18:28:11 */
  /* 012345678901234567 */
  struct tm tm;
  time_t here;
  char *hope = "JanFebMarAprMayJunJulAugSepOctNovDec";
  char *chop = g_strndup(p+3,3);
  tm.tm_year = 2000 + atoi(p+7) - 1900;
  tm.tm_mon = (strstr(hope, chop) - hope) / 3;
  g_free(chop);
  tm.tm_mday = atoi(p);
  tm.tm_hour = atoi(p+10);
  tm.tm_min = atoi(p+13);
  tm.tm_sec = atoi(p+16);
  here = mktime(&tm);
  /* test logic */
  /* fprintf(stderr, "%s --> %d %d %d %d %d %d --> %s", p, tm.tm_year, tm.tm_mon+1, tm.tm_mday, tm.tm_hour, tm.tm_min, tm.tm_sec, ctime(&here)); */
  return here;
}

char *to_name(time_t now)
{
  struct tm *tm = localtime(&now);
  char *name = g_strdup_printf("%04d-%02d-%02d", tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday);
  return name;
}

int main ( int argc, char *argv[] )
{
  char *device;

  /* say g'day */
  fprintf(stderr, "Telephone Call Logger Daemon, 1.0\n"
	 "Copyright (C) 2001  James Cameron (quozl@us.netrek.org)\n"
	 "%s\n\n"
"This program comes with ABSOLUTELY NO WARRANTY; for details see source.\n"
"This is free software, and you are welcome to redistribute it under certain\n"
"conditions; see source for details.\n\n", vcid);

  /* allow user to specify alternate serial port as first argument */
  device = "/dev/ttyS1";
  if (argc > 1) {
    device = argv[1];
    fprintf(stderr, "Selected input serial port %s\n", device);
  }

  /* open the serial port */
  serial_open(device);

#define MAXTIME 32	/* maximum width of timestamp	*/
#define MAXDATA 128	/* maximum width of DTMF data	*/

  /* infinite loop */
  for(;;) {
    time_t t0, t1, h0, h1, tx;
    int byte, n;
    char *p, time_0[MAXTIME], data[MAXDATA], time_1[MAXTIME], *name;
    FILE *output = NULL;

    /* wait for next message */
    byte = fgetc(serial);
    if (byte != '\r') {
      fprintf(stderr, "k164d: state 0 unexpected character %02x\n", byte);
      continue;
    }

    byte = fgetc(serial);
    if (byte != '\n') {
      fprintf(stderr, "k164d: state 1 unexpected character %02x\n", byte);
      continue;
    }

    t0 = time(NULL);

    /* read the date and time stamp dd-mmm-yy hh:mm:ss */
    p = time_0; n = 0;
    while((byte = fgetc(serial)) != ',') {
      if (n++ < MAXTIME) *p++ = byte;
    }
    *p = '\0';
    h0 = to_time(time_0);

    /* create a file name */
    name = to_name(h0);
    output = fopen(name, "a");
    if (output == NULL) { perror(name); exit(1); }

    /* begin the output */
    dputs(time_0, output);
    dputc(',', output);
    dputt(h0, output);
    dputc(',', output);
    dflush(output);

    /* read the DTMF data */
    p = data; n = 0; tx = time(NULL);
    while((byte = fgetc(serial)) != ',') {
      time_t tn = time(NULL);
      if ((tn - tx) > 1) dputc('=', output);
      tx = tn;
      if (n++ < MAXDATA) *p++ = byte;
      dputc(byte, output);
      dflush(output);
    }
    *p = '\0';

    dputc(',', output);
    dputs(data, output);
    dputc(',', output);

    t1 = time(NULL);

    /* read the date and time stamp dd-mmm-yy hh:mm:ss */
    p = time_1; n = 0;
    while((byte = fgetc(serial)) != '\r') {
      if (n++ < MAXTIME) *p++ = byte;
    }
    *p = '\0';
    h1 = to_time(time_1);

    dputs(time_1, output);
    dputc(',', output);
    dputt(h1, output);

    /* emit duraction calculated from serial data */
    p = g_strdup_printf(",%d", (int) (h1 - h0));
    dputs(p, output);
    g_free(p);

    /* emit error calculated from our timing */
    p = g_strdup_printf(",%d\n", (int) (h1 - h0) - (int) (t1 - t0));
    dputs(p, output);
    g_free(p);

    dflush(output);
    fclose(output);
    g_free(name);

    /* read terminating newline */
    byte = fgetc(serial);
    if (byte != '\n') {
      fprintf(stderr, "k164d: state 2 unexpected character %02x\n", byte);
      continue;
    }
  }
}
